#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<cuda.h>
#include<cuda_runtime.h>

__global__ void prime_check(int *a, int *b, int *c, int *A, int *Q, int *R, int n, int pnum)
{
	int index = threadIdx.x + blockIdx.x*blockDim.x;

	a[ index] = threadIdx.x;
	b[ index] = blockDim.x;
	c[ index] = blockIdx.x;

	if (index < n)
	{
		Q[ index] = pnum/A[ index];
		R[ index] = pnum - A[ index]*Q[ index];
	}
}


extern "C" void CUDA_code(int *a, int *b, int *c, int *A, int *Q, int *R, int size, int sqrtP, int P, int M, int N )
{
	int *d_a, *d_b, *d_c, *d_A, *d_Q, *d_R;
	
	printf("\nN = %d, sqrtP = %d, blockDim = %d, and the number of bytes in each array will be %d.\n", P, sqrtP, M, size);

	cudaMalloc((void **)&d_A, size);
	cudaMalloc((void **)&d_Q, size);
	cudaMalloc((void **)&d_R, size);

	cudaMalloc((void **)&d_a, size);
	cudaMalloc((void **)&d_b, size);
	cudaMalloc((void **)&d_c, size);

	cudaMemcpy(d_A, A, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_Q, Q, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_R, R, size, cudaMemcpyHostToDevice);

	cudaMemcpy(d_a, a, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_b, b, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_c, c, size, cudaMemcpyHostToDevice);

	prime_check<<<M, N>>>(d_a, d_b, d_c, d_A, d_Q, d_R, sqrtP, P);

	cudaMemcpy(Q, d_Q, size, cudaMemcpyDeviceToHost);
	cudaMemcpy(R, d_R, size, cudaMemcpyDeviceToHost);

	cudaMemcpy(a, d_a, size, cudaMemcpyDeviceToHost);
	cudaMemcpy(b, d_b, size, cudaMemcpyDeviceToHost);
	cudaMemcpy(c, d_c, size, cudaMemcpyDeviceToHost);
	
	cudaFree(d_A);
	cudaFree(d_Q);
	cudaFree(d_R);

	cudaFree(d_a);
	cudaFree(d_b);
	cudaFree(d_c);
}
