#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<mpi.h>

int main(int argc, char ** argv)
{

	int my_rank, world_size;
	MPI_Status status;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
	MPI_Comm_size(MPI_COMM_WORLD, &world_size);

	int P = atoi(argv[1]); //  Number to test for primality
	int N = atoi(argv[2]); //  Will be number of threads for add<<<1,N>>>, with N < 513

	int sqrtP = sqrt(P); // size of array needed to test for primality
	int M = (sqrtP+(N-1))/N; // block dimension

	int i;
	int *a, *b, *c, *A, *Q, *R;
	
//	num = 1000;

	int size = sqrtP*sizeof(int);

	printf("\nN = %d, sqrtP = %d, blockDim = %d, and the number of bytes in each array will be %d.\n", P, sqrtP, M, size);

	A = (int *)malloc(size); // Will have integers from 2 to sqrtP+2
	Q = (int *)malloc(size); // Will store quotient of dividing num by A[i]
	R = (int *)malloc(size); // Will store remainder of dividing num by A[i]

	a = (int *)malloc(size); // stores thread index
	b = (int *)malloc(size); // stores block dimension
	c = (int *)malloc(size); // stores block index


	for (i = 0; i < sqrtP; i++) 
	{
		a[i] = 0;
		b[i] = 0;
		c[i] = 0;
		A[i] = i+2; // start the primality check with 2 
		Q[i] = 0;
		R[i] = -1; // set all remainders to -1 to make sure they are computed correctly
	}

	CUDA_code(a, b, c, A, Q, R, size, sqrtP, P, M, N);
	
	printf("\n");

	for (i = 0; i < sqrtP; i++) 
	{
		if (R[i] < 1)
		{
			printf("i = %d: number = %d, divisor = %d, quotient = %d, remainder = %d, threadIdx = %d, blockDim = %d, blockIdx = %d\n", i, P, A[i], Q[i], R[i], a[i], b[i], c[i]);
		}	
	}
	
	free(A);
	free(Q);
	free(R);

	free(a);
	free(b);
	free(c);

	MPI_Finalize();

	return 0;

}
